# -*- coding: utf-8 -*-
"""
Create a cert with pyOpenSSL for tests.

Heavily based on python-opsi's OPSI.Util.Task.Certificate.
Source: https://github.com/opsi-org/python-opsi/blob/stable/OPSI/Util/Task/Certificate.py
"""
import argparse
import os
import random
import socket
from tempfile import NamedTemporaryFile

from OpenSSL import crypto

try:
	import secrets
except ImportError:
	secrets = None


def createCertificate(path):
	"""
	Creates a certificate.
	"""
	cert = crypto.X509()
	cert.get_subject().C = "DE"  # Country
	cert.get_subject().ST = "HE"  # State
	cert.get_subject().L = "Wiesbaden"  # Locality
	cert.get_subject().O = "pytest-tornado"  # Organisation
	cert.get_subject().OU = "Testing Department"  # organisational unit
	cert.get_subject().CN = socket.getfqdn()  # common name

	# As described in RFC5280 this value is required and must be a
	# positive and unique integer.
	# Source: http://tools.ietf.org/html/rfc5280#page-19
	cert.set_serial_number(random.randint(0, pow(2, 16)))

	cert.gmtime_adj_notBefore(0)
	cert.gmtime_adj_notAfter(60 * 60)  # Valid 1 hour

	k = crypto.PKey()
	k.generate_key(crypto.TYPE_RSA, 2048)

	cert.set_issuer(cert.get_subject())
	cert.set_pubkey(k)
	cert.set_version(2)

	cert.sign(k, 'sha512')

	certcontext = b"".join(
		(
			crypto.dump_certificate(crypto.FILETYPE_PEM, cert),
			crypto.dump_privatekey(crypto.FILETYPE_PEM, k)
		)
	)

	with open(path, "wt") as certfile:
		certfile.write(certcontext.decode())

	try:
		with NamedTemporaryFile(mode="wb", delete=False) as randfile:
			randfile.write(randomBytes(512))

			command = u"openssl dhparam -rand {tempfile} 512 >> {target}".format(
				tempfile=randfile.name, target=path
			)
		os.system(command)
	finally:
		os.remove(randfile.name)


def randomBytes(length):
	"""
	Return _length_ random bytes.

	:rtype: bytes
	"""
	if secrets:
		return secrets.token_bytes(512)
	else:
		return os.urandom(512)


if __name__ == '__main__':
	parser = argparse.ArgumentParser(description='Create certificate for testing')
	parser.add_argument('--cert', dest='cert', default="testcert.pem",
	                    help='Name of the certificate')

	args = parser.parse_args()
	createCertificate(args.cert)



import functools
import pytest
import tornado.ioloop
import tornado.web


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.write('Hello, world')


application = tornado.web.Application([
    (r'/', MainHandler),
    (r'/f00', MainHandler),
])


@pytest.fixture(scope='module')
def app():
    return application


def _fetch(http_client, url):
    return http_client.io_loop.run_sync(
        functools.partial(http_client.fetch, url))


def test_http_server(http_server, io_loop):
    status = {'done': False}

    def _done():
        status['done'] = True
        io_loop.stop()

    io_loop.add_callback(_done)
    io_loop.start()

    assert status['done']


def test_http_client(http_client, base_url):
    request = http_client.fetch(base_url)
    request.add_done_callback(lambda future: http_client.io_loop.stop())
    http_client.io_loop.start()

    response = request.result()
    assert response.code == 200


def test_http_client_with_fetch_helper(http_client, base_url):
    response = _fetch(http_client, base_url)
    assert response.code == 200


@pytest.mark.gen_test
def test_http_client_with_gen_test(http_client, base_url):
    response = yield http_client.fetch(base_url)
    assert response.code == 200


@pytest.mark.gen_test
def test_get_url_with_path(http_client, base_url):
    response = yield http_client.fetch('%s/f00' % base_url)
    assert response.code == 200


@pytest.mark.gen_test
def test_http_client_raises_on_404(http_client, base_url):
    with pytest.raises(tornado.httpclient.HTTPError):
        yield http_client.fetch('%s/bar' % base_url)


def test_base_url_default_is_http(base_url):
    assert base_url.startswith('http://')


import os
import ssl
import pytest
import tornado.web


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.write("Hello, world")


@pytest.fixture
def ssl_options():
    cert_file = os.path.join(os.path.dirname(__file__), 'testcert.pem')
    if not os.path.exists(cert_file):
        pytest.skip("Missing cert file {!r}")
    key_file = os.path.join(os.path.dirname(__file__), 'testcert.pem')
    if not os.path.exists(key_file):
        pytest.skip("Missing key file {!r}")

    ssl_context = ssl.create_default_context(purpose=ssl.Purpose.CLIENT_AUTH, cafile=None, capath=None, cadata=None)
    ssl_context.load_cert_chain(cert_file, keyfile=key_file)
    return ssl_context


@pytest.fixture
def app():
    return tornado.web.Application([
        (r"/", MainHandler),
    ])


@pytest.mark.gen_test
def test_hello_world(https_client, base_url):
    response = yield https_client.fetch(base_url, validate_cert=False)
    assert response.code == 200


def test_base_url_is_https_with_https_client(https_client, base_url):
    assert base_url.startswith('https://')



def test_base_url_is_https_with_https_port(https_port, base_url):
    assert base_url.startswith('https://')



def test_base_url_is_https_with_https_server(https_server, base_url):
    assert base_url.startswith('https://')



import pytest
import sys
from tornado import gen

_used_fixture = False


@gen.coroutine
def dummy(io_loop):
    yield gen.sleep(0)
    raise gen.Return(True)


@pytest.fixture(scope='module')
def preparations():
    global _used_fixture
    _used_fixture = True


pytestmark = pytest.mark.usefixtures('preparations')


@pytest.mark.xfail(pytest.__version__ < '2.7.0',
                   reason='py.test 2.7 adds hookwrapper, fixes collection')
@pytest.mark.gen_test
def test_uses_pytestmark_fixtures(io_loop):
    assert (yield dummy(io_loop))
    assert _used_fixture

class TestClass:
    def beforeEach(self):
        global _used_fixture
        _used_fixture = False

    @pytest.mark.gen_test
    def test_uses_pytestmark_fixtures(self, io_loop):
        assert (yield dummy(io_loop))
        assert _used_fixture


@pytest.mark.xfail(sys.version_info < (3, 5),
                   reason='Type hints added in Python 3.5')
def test_type_annotation(testdir):

    testdir.makepyfile(
        test_type_annotation="""
            import pytest
            from tornado.ioloop import IOLoop

            @pytest.mark.gen_test
            def test_type_attrib(io_loop: IOLoop):
                pass  # Only check that gen_test works
        """,
    )

    # Run tests
    result = testdir.runpytest_inprocess()

    # Check tests went off as they should:
    assert result.ret == 0



import functools
import pytest
import tornado
from tornado import gen
from tornado.ioloop import TimeoutError


@gen.coroutine
def dummy_coroutine(io_loop):
    yield gen.sleep(0)
    raise gen.Return(True)


def test_explicit_start_and_stop(io_loop):
    future = dummy_coroutine(io_loop)
    future.add_done_callback(lambda *args: io_loop.stop())
    io_loop.start()
    assert future.result()


def test_run_sync(io_loop):
    dummy = functools.partial(dummy_coroutine, io_loop)
    finished = io_loop.run_sync(dummy)
    assert finished


@pytest.mark.gen_test
def test_gen_test_sync(io_loop):
    assert True


@pytest.mark.gen_test
def test_gen_test(io_loop):
    result = yield dummy_coroutine(io_loop)
    assert result


@pytest.mark.gen_test(run_sync=False)
def test_gen_test_run_sync_false(io_loop):
    result = yield dummy_coroutine(io_loop)
    assert result


@pytest.mark.gen_test
def test_gen_test_swallows_exceptions(io_loop):
    with pytest.raises(ZeroDivisionError):
        1 / 0


@pytest.mark.gen_test
def test_generator_raises(io_loop):
    with pytest.raises(ZeroDivisionError):
        yield gen.sleep(0)
        1 / 0


@pytest.mark.gen_test
def test_explicit_gen_test_marker(request, io_loop):
    yield gen.sleep(0)
    assert 'gen_test' in request.keywords


@pytest.mark.gen_test(timeout=2.1)
def test_gen_test_marker_with_params(request, io_loop):
    yield gen.sleep(0)
    assert request.node.get_closest_marker('gen_test').kwargs['timeout'] == 2.1


@pytest.mark.xfail(raises=TimeoutError)
@pytest.mark.gen_test(timeout=0.1)
def test_gen_test_with_timeout(io_loop):
    yield gen.sleep(1)


def test_sync_tests_no_gen_test_marker(request):
    assert 'gen_test' not in request.keywords


class TestClass:
    @pytest.mark.gen_test
    def test_gen_test(self, io_loop):
        result = yield dummy_coroutine(io_loop)
        assert result

    @pytest.mark.gen_test
    def test_generator_raises(self, io_loop):
        with pytest.raises(ZeroDivisionError):
            yield gen.sleep(0)
            1 / 0



import pytest
from tornado import gen

DUMMY_PARAMS = ['f00', 'bar']


@pytest.fixture(params=DUMMY_PARAMS)
def _dummy(request):
    return request.param


@pytest.mark.parametrize('input,expected', [
    ('3+5', 8),
    ('2+4', 6),
])
def test_eval(input, expected):
    assert eval(input) == expected


@pytest.mark.parametrize('input,expected', [
    ('3+5', 8),
    ('2+4', 6),
    pytest.param("6*9", 42,
                 marks=pytest.mark.xfail),
])
def test_eval_marking(input, expected):
    assert eval(input) == expected


@pytest.mark.parametrize('input,expected', [
    ('3+5', 8),
    ('2+4', 6),
])
@pytest.mark.gen_test
def test_sync_eval_with_gen_test(input, expected):
    assert eval(input) == expected


@pytest.mark.parametrize('input,expected', [
    ('3+5', 8),
    ('2+4', 6),
])
def test_eval_with_fixtures(input, io_loop, expected):
    assert eval(input) == expected


def test_param_fixture(_dummy):
    assert _dummy in DUMMY_PARAMS


@pytest.mark.gen_test
@pytest.mark.parametrize('input,expected', [
    ('3+5', 8),
    ('2+4', 6),
])
def test_gen_test_parametrize(io_loop, input, expected):
    yield gen.sleep(0)
    assert eval(input) == expected


@pytest.mark.parametrize('input,expected', [
    ('3+5', 8),
    ('2+4', 6),
])
@pytest.mark.gen_test
def test_gen_test_fixture_any_order(input, io_loop, expected):
    yield gen.sleep(0)
    assert eval(input) == expected


@pytest.mark.gen_test
def test_gen_test_param_fixture(io_loop, _dummy):
    yield gen.sleep(0)
    assert _dummy in DUMMY_PARAMS



import os
import io
from setuptools import setup, find_packages


cwd = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(cwd, 'README.rst'), encoding='utf-8') as fd:
    long_description = fd.read()


setup(
    name='pytest-tornado',
    version='0.8.1',
    description=('A py.test plugin providing fixtures and markers '
                 'to simplify testing of asynchronous tornado applications.'),
    long_description=long_description,
    url='https://github.com/eugeniy/pytest-tornado',
    author='Eugeniy Kalinin',
    author_email='burump@gmail.com',
    maintainer='Vidar Tonaas Fauske',
    maintainer_email='vidartf@gmail.com',
    license='Apache License, Version 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        'Topic :: Software Development :: Testing',
    ],
    keywords=('pytest py.test tornado async asynchronous '
              'testing unit tests plugin'),
    packages=find_packages(exclude=["tests.*", "tests"]),
    install_requires=['pytest>=3.6', 'tornado>=4.1', 'setuptools'],
    entry_points={
        'pytest11': ['tornado = pytest_tornado.plugin'],
    },
)



import pytest
from tornado import gen

async def dummy_native_coroutine(io_loop):
    await gen.sleep(0)
    return True


@pytest.mark.gen_test
async def test_native_coroutine_gen_test(io_loop):
    result = await dummy_native_coroutine(io_loop)
    assert result


@pytest.mark.gen_test(run_sync=False)
async def test_native_coroutine_run_sync_false(io_loop):
    result = await dummy_native_coroutine(io_loop)
    assert result
